----------------------------------------------------------
-- Chat and speech bubbles
----------------------------------------------------------

local textboxcheck
local newmessage
local function ChatFilterHandler(self, event, message, sender, ...)
	local originalmessage=message
	if(sender=="Chromie")then
		message=ChromieTime(message)
	end
	
	-- if the message type might have produced a speech bubble, start scanning for that too
	if(message~=originalmessage)then
		if(event=="CHAT_MSG_SAY" 
		or event=="CHAT_MSG_PARTY"	
		or event=="CHAT_MSG_PARTY_LEADER"
		or event=="CHAT_MSG_YELL"
		or event=="CHAT_MSG_MONSTER_SAY" 
		or event=="CHAT_MSG_MONSTER_PARTY"
		or event=="CHAT_MSG_MONSTER_YELL"
		or event=="CHAT_MSG_BOSS_EMOTE"
		)then
			textboxcheck=originalmessage
			newmessage=message
		end
	end
	return false, message, sender, ...
end


--SOURCE: us.battle.net/forums/en/wow/topic/14058435776
local function ScanChildren(...)
	for i=1,select("#",...) do
		local frame = select(i,...)
		if(not frame:IsForbidden())then

			local regions=frame:GetRegions()
			local numregions=frame:GetNumRegions()
			if not frame:GetName() and numregions  then
				--print("Possible chat bubble found:",frame,frame.name)
				RREGIONS=regions
				local selectframe=select(11,frame:GetRegions())
				if(selectframe.GetText)then
					local contents=selectframe:GetText()
					--print("Contents:",contents,"(comparing to",textboxcheck..")")
					if(contents==textboxcheck)then
						textboxcheck=nil		--found it!	 stop scanning

						local newtext=newmessage
						selectframe:SetText(newtext)
						
						local newwidth=string.len(newtext)*9
						if(newwidth>300)then newwidth=300 end
						selectframe:SetWidth(newwidth)
						local parent=frame:GetParent()			
end	end	end	end end end

local timer=0
local f = CreateFrame("Frame")
--TODO: we can probably replace this with a C_Timer function that only runs when we're expecting a textbox
f:SetScript("OnUpdate",function(self,elapsed)
	if(textboxcheck)then
		timer = timer + elapsed
		if timer > 0.125 then
			timer = 0
			ScanChildren(WorldFrame:GetChildren())
end end end)

-- We can't randomize chromiecounter (the variable that checks word frequency) EVERY time we run ChromieTime 
-- or else the result won't be consistent across multiple chat windows.
-- Instead, randomize it once every couple seconds.
local chromiecounter_start=0
C_Timer.NewTicker(2,function()
	local frequency=1
	if(CHROMIETIME_VARS)then frequency=CHROMIETIME_VARS.frequency end
	chromiecounter_start=math.floor(math.random()*frequency)
end)

ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM",ChatFilterHandler); 	-- whispers sent from self to other
ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM",ChatFilterHandler);
ChatFrame_AddMessageEventFilter("CHAT_MSG_SAY",ChatFilterHandler);
ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY",ChatFilterHandler);
ChatFrame_AddMessageEventFilter("CHAT_MSG_PARTY_LEADER",ChatFilterHandler);
ChatFrame_AddMessageEventFilter("CHAT_MSG_INSTANCE_CHAT",ChatFilterHandler);
ChatFrame_AddMessageEventFilter("CHAT_MSG_INSTANCE_CHAT_LEADER",ChatFilterHandler);
ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID",ChatFilterHandler);
ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_LEADER",ChatFilterHandler);
ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_WARNING",ChatFilterHandler);
ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL",ChatFilterHandler);
ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER",ChatFilterHandler);
ChatFrame_AddMessageEventFilter("CHAT_MSG_GUILD",ChatFilterHandler);
ChatFrame_AddMessageEventFilter("CHAT_MSG_YELL",ChatFilterHandler);
ChatFrame_AddMessageEventFilter("CHAT_MSG_EMOTE",ChatFilterHandler);
ChatFrame_AddMessageEventFilter("CHAT_MSG_TEXT_EMOTE",ChatFilterHandler);
ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL",ChatFilterHandler);
ChatFrame_AddMessageEventFilter("CHAT_MSG_BOSS_EMOTE",ChatFilterHandler);
ChatFrame_AddMessageEventFilter("CHAT_MSG_MONSTER_EMOTE",ChatFilterHandler);
ChatFrame_AddMessageEventFilter("CHAT_MSG_MONSTER_SAY",ChatFilterHandler);
ChatFrame_AddMessageEventFilter("CHAT_MSG_MONSTER_WHISPER",ChatFilterHandler);
ChatFrame_AddMessageEventFilter("CHAT_MSG_MONSTER_PARTY",ChatFilterHandler);
ChatFrame_AddMessageEventFilter("CHAT_MSG_MONSTER_YELL",ChatFilterHandler);




----------------------------------------------------------
-- Gossip, quests, and other frames
----------------------------------------------------------

-- calling SetText from within the SetText hook function requires a bit of effort
-- to avoid getting caught in an infinite loop.
local ignore_text=nil
local function SetText_Hook(self,text)
	--print("Called SetText_Hook from",self:GetName())
	--print("ChromieTime ignore_text:",ignore_text)
	if(self==GameTooltip)then print("SetText_Hook",text) end
	if(self:IsForbidden())then return end	
	if(text)then
		if(text~=ignore_text)then
			local unitname=UnitName("npc")
			local newtext=text
			if(unitname=="Chromie")then
				newtext=ChromieTime(text)
			end
			if(newtext~=text)then
				ignore_text=newtext
				self:SetText(newtext)
			end
		else
			--print("Ignored",self:GetName())
		end
	end	
end
hooksecurefunc(GossipFrameNpcNameText,"SetText",SetText_Hook)
hooksecurefunc(GossipGreetingText,"SetText",SetText_Hook)
hooksecurefunc(QuestFrameNpcNameText,"SetText",SetText_Hook)
hooksecurefunc(QuestInfoDescriptionText,"SetText",SetText_Hook)
hooksecurefunc(QuestProgressText,"SetText",SetText_Hook)
hooksecurefunc(QuestInfoRewardText,"SetText",SetText_Hook)



----------------------------------------------------------
-- Slash commands
----------------------------------------------------------

-- whenever we run the ChromieTime translator, save a copy of the raw text
-- in case the player needs to read it later
local unaltered_text=nil

SLASH_CHROMIETIME1 = "/chromietime";
function SlashCmdList.CHROMIETIME(msg,editbox)
	if(not CHROMIETIME_VARS)then
		CHROMIETIME_VARS={}
		CHROMIETIME_VARS.frequency=2
	end

	local args={};
	local i=0;
	for arg in string.gmatch(msg, "%S+") do	
		args[i]=arg;
		i=i+1;
	end
	if(args[0]=="translate") then
		if(unaltered_text)then
			print("Chromie said:",unaltered_text)
		else
			print("Chromie hasn't said anything yet.")
		end
	elseif(args[0]=="frequency") then
		if(CHROMIETIME_VARS.frequency==2)then
			CHROMIETIME_VARS.frequency=1
			print("Frequency toggled.  ChromieTime will now alter almost ALL of Chromie's words.")
		else
			CHROMIETIME_VARS.frequency=2
			print("Frequency toggled.  ChromieTime will now alter no more than HALF of Chromie's words.")
		end
	else
		print("ChromieTime slash commands:")
		print("/chromietime frequency")
		print("-- Toggle how often ChromieTime changes words.")
		print("/chromietime translate")
		print("-- Repeat the last thing Chromie said, unaltered.")
	end
end


----------------------------------------------------------
-- Text substitution
----------------------------------------------------------

-- 1-2 letter words are always filtered 
-- manually filter a few other short words and words relating to time
local chromiewords={"i'm","i've","you","they","them","out","hour","hours","minute","minutes","second","seconds","past","present","now","future","when","freeze","stop","next","day","days","skip","portal","timestream","chromie"}
--TODO: add bronze dragon name endings
function ChromieTime(text)

	-- Store unaltered text in case the player needs to read it later.
	-- First, check that we're not calling ChromieTime on Chromie's name rather than her text.
	if(text~="Chromie")then
		unaltered_text=text
	end
	
	
	if(not CHROMIETIME_VARS)then
		CHROMIETIME_VARS={}
		CHROMIETIME_VARS.frequency=2
	end
	
	local result=""		
	local chromiecounter=chromiecounter_start
	local splits=SuperNaiveSplit(text)		
	for i=1,#splits do
		local newword=splits[i]		
		if(not IsAWord(splits[i]))then
			-- if word is space or punctuation, skip it
		else			
			if(string.len(splits[i])<=2)then
				-- if word is < 3 letters, don't change it			
			elseif(TRAININGGROUNDS_tcontains(chromiewords,string.lower(splits[i])))then
				--if word is in list of chromiewords, don't change it
			else
				chromiecounter=chromiecounter+1		
				if(chromiecounter>=CHROMIETIME_VARS.frequency)then				
					newword="time"
					--check chromiesuffixes individually
					local lowersplit=string.lower(splits[i])
					local l=string.len(lowersplit)					
					if(string.sub(lowersplit,l-2)=="'ve")then
						newword="time've"
					elseif(string.sub(lowersplit,l-1)=="'d")then
						newword="time'd"
					elseif(string.sub(lowersplit,l-1)=="ly")then
						newword="timely"
					elseif(string.sub(lowersplit,l-0)=="y")then
						newword="timey"
					elseif(string.sub(lowersplit,l-0)=="d")then
						newword="timed"	
					elseif(string.sub(lowersplit,l-2)=="ing")then
						newword="timing"
					elseif(string.sub(lowersplit,l-3)=="ings")then
						newword="timings"
					elseif(string.sub(lowersplit,l-2)=="'ll")then
						newword="time'll"
					elseif(string.sub(lowersplit,l-1)=="'s")then
						newword="time's"
					elseif(string.sub(lowersplit,l-0)=="s")then
						newword="times"	
					elseif(string.sub(lowersplit,l-0)=="r")then
						newword="timer"						
					elseif(string.sub(lowersplit,l-2)=="n't")then
						newword="timen't"						
					end				
								
					--if whole word was capitalized, same for new word
					if(string.upper(splits[i])==splits[i])then
						newword=string.upper(newword)
					--if first letter was capitalized, same for new word
					elseif(string.upper(string.sub(splits[i],1,1))==string.sub(splits[i],1,1))then
						newword=string.upper(string.sub(newword,1,1))..string.sub(newword,2)
					end			
					--print("Input:",splits[i],"Output:",newword,"(reset chromiecounter)")
					chromiecounter=0
				else
					--print("Input:",splits[i],"(skip)")
				end				
			end			
		end

		result=result..newword
	end
	
	return result
end

function SuperNaiveSplit(text)
	-- look through text character by character, 
	-- splitting words (and whitespace) whenever we encounter something that isn't a letter or apostrophe.
	-- there's probably a much easier way to do this using string.split if we were fluent in regex.
	local lettermode=true
	local results={}
	results[1]=""
	local index=1
	for i=1,string.len(text) do
		local isletter
		if(string.find(string.sub(text,i,i),"[%a']"))then 
			isletter=true
		else
			isletter=false
		end
		if(isletter==lettermode)then				
			-- add current letter/whitespace to the existing word
			results[index]=results[index]..string.sub(text,i,i)
		else
			-- start new word
			lettermode=isletter
			index=index+1
			results[index]=string.sub(text,i,i)
		end
	end
	return results
end

function IsAWord(word)
	if(string.find(word,"%a"))then 
		return true
	else 
		return false 
	end
end

